import pytest

from uwsgi_prometheus.collectors import UWSGIStatsCollector


@pytest.fixture
def mock_uwsgi_stats(mocker):
    response = mocker.Mock(status_code=200, json=lambda: {
        'listen_queue': 2,
        'workers': [
            {
                'requests': 10,
                'harakiri_count': 1,
            },
            {
                'requests': 5,
                'harakiri_count': 2,
            }
        ]
    })
    return mocker.patch('requests.get', return_value=response)


def test_default_collector(mock_uwsgi_stats):
    collector = UWSGIStatsCollector()
    metrics = list(collector.collect())

    assert metrics[0].name == 'uwsgi_listen_queue_requests'
    assert metrics[0].samples[0].name == 'uwsgi_listen_queue_requests'
    assert metrics[0].samples[0].value == 2

    # Prometheus does interesting things with names that end with certain words, like total
    assert metrics[1].name == 'uwsgi_requests'
    assert metrics[1].samples[0].name == 'uwsgi_requests_total'
    assert metrics[1].samples[0].value == 15

    assert metrics[2].name == 'uwsgi_harakiris'
    assert metrics[2].samples[0].name == 'uwsgi_harakiris_total'
    assert metrics[2].samples[0].value == 3

    with pytest.raises(IndexError):
        assert metrics[3].name == 'shouldnt_exist'

    mock_uwsgi_stats.assert_called_once_with('http://127.0.0.1:1717', timeout=2)


def test_collector_with_options(mock_uwsgi_stats):
    collector = UWSGIStatsCollector(stats_url='http://192.168.0.1:1919', timeout=5, prefix='test')
    metrics = list(collector.collect())

    assert metrics[0].name == 'test_uwsgi_listen_queue_requests'
    assert metrics[0].samples[0].name == 'test_uwsgi_listen_queue_requests'
    assert metrics[0].samples[0].value == 2

    # Prometheus does interesting things with names that end with certain words, like total
    assert metrics[1].name == 'test_uwsgi_requests'
    assert metrics[1].samples[0].name == 'test_uwsgi_requests_total'
    assert metrics[1].samples[0].value == 15

    assert metrics[2].name == 'test_uwsgi_harakiris'
    assert metrics[2].samples[0].name == 'test_uwsgi_harakiris_total'
    assert metrics[2].samples[0].value == 3

    with pytest.raises(IndexError):
        assert metrics[3].name == 'shouldnt_exist'

    mock_uwsgi_stats.assert_called_once_with('http://192.168.0.1:1919', timeout=5)



import requests

from prometheus_client.core import GaugeMetricFamily, CounterMetricFamily


class UWSGIStatsCollector:
    def __init__(
        self,
        stats_url='http://127.0.0.1:1717',
        timeout=2,
        prefix='',
    ):
        self.stats_url = stats_url
        self.timeout = timeout
        self.prefix = '%s_' % prefix if prefix else prefix

    def collect(self):
        try:
            response = requests.get(self.stats_url, timeout=self.timeout)
            response.raise_for_status()
        except (requests.ConnectionError, requests.HTTPError):
            return

        uwsgi_stats = response.json()

        listen_queue_length = uwsgi_stats['listen_queue']
        requests_total = 0
        harakiris_total = 0
        for worker in uwsgi_stats['workers']:
            requests_total += worker['requests']
            harakiris_total += worker['harakiri_count']

        yield GaugeMetricFamily('%suwsgi_listen_queue_requests' % self.prefix, 'Number of requests in the uWSGI listen queue', value=listen_queue_length)
        yield CounterMetricFamily('%suwsgi_requests_total' % self.prefix, 'Total number of uWSGI requests', value=requests_total)
        yield CounterMetricFamily('%suwsgi_harakiris_total' % self.prefix, 'Total number of Harakiris', value=harakiris_total)



import os
import re
from io import open

from setuptools import find_packages, setup


def read(file):
    with open(file, 'r', encoding='utf-8') as f:
        return f.read()


def get_version():
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    init_py = open(os.path.join('uwsgi_prometheus', '__init__.py')).read()
    return re.search("__version__ = ['\"]([^'\"]+)['\"]", init_py).group(1)


version = get_version()


setup(
    name='uwsgi-prometheus',
    version=version,
    url='https://github.com/GlobalRadio/uwsgi-prometheus',
    license='MIT',
    description='uWSGI stats exporter for Prometheus',
    long_description=read('README.md'),
    long_description_content_type='text/markdown',
    author='Chris Graham',
    author_email='chris.graham@global.com',
    packages=find_packages(),
    include_package_data=True,
    python_requires=">=3.8",
    install_requires=[
        'prometheus-client',
        'requests',
    ],
    zip_safe=False,
    classifiers=[
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
    ],
    tests_require=[
        'pytest>=7.0.0,<8.0.0',
        'pytest-cov',
        'pytest-mock>=3.0.0',
    ],
)
